# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import json
from unittest import TestCase

import six
from mock import patch

from .base import BaseTestCase, get_client
from .role_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestRoleGet(TestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_role_assertions(self, mock_get, mock_post):

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            get_client().role.get(123)

        # pylint: disable=anomalous-backslash-in-string
        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v1/role/",
        ):
            get_client().role.get("garbage")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_role(self, mock_get, mock_post):

        role = get_client().role.get("/api/v1/role/1")

        self.assertEqual(role.id, "/api/v1/role/1")
        self.assertEqual(role.name, "Site Admin")
        self.assertEqual(role.description, "Role for Site Admin")


class TestCLI(BaseTestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get(self, mock_post, mock_delete):

        try:
            hpecp = self.cli.CLI()
            hpecp.role.get("/api/v1/role/1")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = """\
_links:
  all_roles:
    href: /api/v1/role
  self:
    href: /api/v1/role/1
label:
  description: Role for Site Admin
  name: Site Admin"""

        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_json(self, mock_post, mock_delete):

        try:
            hpecp = self.cli.CLI()
            hpecp.role.get("/api/v1/role/1", output="json")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        stdout = json.dumps(json.loads(stdout), sort_keys=True)

        expected_stdout = json.dumps(
            {
                "label": {
                    "description": "Role for Site Admin",
                    "name": "Site Admin",
                },
                "_links": {
                    "all_roles": {"href": "/api/v1/role"},
                    "self": {"href": "/api/v1/role/1"},
                },
            },
            sort_keys=True,
        )

        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import json
import jmespath
import sys
import yaml
from textwrap import dedent

from hpecp.cli import base
from hpecp.cli_utils import TextOutput


class InstallProxy(object):
    """Proxy object to :py:attr:`<hpecp.client.install>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["examples", "get", "set_gateway_ssl"]

    @base.intercept_exception
    def get(self, output="yaml", query=None):
        """Get the system config.

        :param output: how to display the output '[yaml|json|json-pp]'
        """
        if output not in ["yaml", "json", "json-pp", "text"]:
            print(
                (
                    "'output' parameter must be 'yaml', 'json', "
                    "'json-pp' or 'text'."
                ),
                file=sys.stderr,
            )
            sys.exit(1)

        if output == "yaml" and query is not None:
            print(
                "output=yaml is not supported with the query parameter.",
                file=sys.stderr,
            )
            sys.exit(1)

        response = base.get_client().install.get()

        if output == "yaml":
            print(
                yaml.dump(
                    yaml.load(
                        json.dumps(response),
                        Loader=yaml.FullLoader,
                    )
                )
            )
        else:
            if query is None:
                data = response
            else:
                data = jmespath.search(str(query), response)

            if output == "json-pp":
                print(json.dumps(data, indent=4, sort_keys=True))
            elif output == "text":
                print(TextOutput.dump(data))
            else:
                print(json.dumps(data))

    def examples(self):
        """Show examples for working with roles."""
        print(
            dedent(
                """\

                """  # noqa:  E501
            )
        )

    @base.intercept_exception
    def set_gateway_ssl(
        self,
        cert_file=None,
        cert_content=None,
        cert_file_name=None,
        key_file=None,
        key_content=None,
        key_file_name=None,
    ):
        """Set Gateway SSL.

        Parameters
        ----------
        cert_file : [type]
            [description]
        cert_content : [type]
            [description]
        cert_file_name : [type]
            [description]
        key_file : [type]
            [description]
        key_content : [type]
            [description]
        key_file_name : [type]
            [description]
        """
        assert (
            cert_file is None
            and (cert_content is not None and cert_file_name is not None)
        ) or (
            cert_file is not None
            and (cert_content is None and cert_file_name is None)
        ), (
            "('cert-content' and 'cert-file-name') or 'cert-file' "
            "must be provided."
        )

        assert (
            key_file is None
            and (key_content is not None and key_file_name is not None)
        ) or (
            key_file is not None
            and (key_content is None and key_file_name is None)
        ), (
            "('key_content' and 'key_file-name') or 'key_file' "
            "must be provided."
        )

        if cert_file:
            try:
                with open(cert_file, "r") as f:
                    cert_content = (
                        f.read().replace("\n", "\n").replace("\t", "\t")
                    )
                cert_file_name = cert_file
            except OSError:
                print(
                    "Could not open/read 'cert-file': {}".format(cert_file),
                    file=sys.stderr,
                )
                sys.exit(1)

        if key_file:
            try:
                with open(key_file, "r") as f:
                    key_content = (
                        f.read().replace("\n", "\n").replace("\t", "\t")
                    )
                key_file_name = key_file
            except OSError:
                print(
                    "Could not open/read 'key-file': {}".format(key_file),
                    file=sys.stderr,
                )
                sys.exit(1)

        base.get_client().install.set_gateway_ssl(
            cert_content, cert_file_name, key_content, key_file_name
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiGetSetup():
    mockApiGetWorkers()
    mockApiGetWorkers97()
    mockApiGetWorkers98()
    mockApiGetWorkers99()
    mockApiGetWorkers100()
    mockApiGetWorkers999()


def mockApiGetWorkers():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v1/workers"}},
                "_embedded": {
                    "workers": [
                        {
                            "_links": {"self": {"href": "/api/v1/workers/1"}},
                            "ip": "10.1.0.223",
                            "purpose": "primary",
                            "state": "installed",
                            "hostname": "ip-10-1-0-223.us-west-2.compute.internal",
                            "sysinfo": {
                                "network": [
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "96:7b:70:f0:80:65",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-6-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "9e:ee:3e:55:4f:5e",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-2-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "f6:40:25:22:e4:9e",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-0-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "UNKNOWN",
                                            "Carrier": "UNKNOWN",
                                            "HwAddr": "72:c9:23:3a:10:4f",
                                            "Speed": "UNKNOWN",
                                        },
                                        "name": "bds-flood",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "56:ee:25:7b:21:d2",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-1-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "fe:a0:8c:a1:48:62",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-1-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "26:cf:9e:07:d2:30",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-0-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "5e:b2:25:ac:0d:a2",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-2-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "86:df:9f:11:4a:26",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-6-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": False,
                                            "IpAddr": {
                                                "dynamic": "10.1.0.223/24"
                                            },
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "02:74:0e:2c:c8:b6",
                                            "Speed": "UNKNOWN",
                                        },
                                        "name": "ens5",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "7a:84:67:be:a7:65",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-4-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "ee:c1:00:ad:ca:f0",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-5-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "e6:16:cf:dc:e0:60",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-3-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "ee:6c:a1:4c:75:1f",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-7-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "0e:76:6b:67:68:d8",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-7-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "f6:c9:f5:d9:a0:af",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-3-r",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "b2:dc:68:f1:b1:a0",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-5-l",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "16:2b:7b:14:15:85",
                                            "Speed": "10000",
                                        },
                                        "name": "bds-flood-4-l",
                                    },
                                ],
                                "keys": {
                                    "reported_worker_public_key": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQDk5SHYy1gwjEtERZNoIDmjgbomLa8psuM5q7+a0LvD3NsPDuvkQ2E2e32QAV5gCHy+yKteADIVK9BxonlQPxe/ArMzbHQjsfFaD+ek0oCr2xCs91wVTrK0LV/kTWLhgaVHDSJAw5U/36dnedMmmarxaV7GtzFVE2uzlIUl1Dr3Zvc7rZCBgLgB3cysVVzfBA9V/L1COl05GTC8jkRErASFX/F/93WbifNyF2+9Nnnnle/RX1Nc/GXaypjSLzGWIXFiuoztUP+DT4k/20izmMqEAF40LNHKpq6bkcjMU8S21boHjo7CtugIkhjwxp+k7dW0ls+SnvKx+3gi9sIFsTZxK7tBYd1dhPzHpl+G57N9YeusUXM4KAi1RGHR7Y1utWFUObfQi0Eh80b3T19gzHVS6AA7MCSbCOzxnWjgVZCMsckCm1YRiTsJ2vx1tcDsoEkl0sqivsF884ahWDZK/UBbx5ZJajrg56Mg27avsGXRc74U1OGRL1smF80OdBBeZ987+cqSqtp7lYDyfARh5CimE62Lljs7INWSQOWMx6ve4gSs/j+dj2e9H7YHXZKR3JvYzuf0J05u5i0jz55Kg2scmHsPgJsg6nMfAImPHtOW+c19olAhT5F5pbCBl9bHcdDgWXFZjjX9uZJCyW0ppLMXgke2qJxjPonG4n1oD49K3Q== server\n"
                                },
                                "storage": [
                                    {
                                        "info": {
                                            "IsDisk": False,
                                            "Name": "/dev/mapper/VolBDSCStore-thinpool",
                                            "IsReadOnly": False,
                                            "IsRemovable": False,
                                            "IsRotational": False,
                                            "DeviceType": "lvm",
                                            "HasFilesystem": True,
                                            "IsLogicalVolume": True,
                                            "IsPartition": False,
                                            "ParentName": "/dev/nvme2n1p1",
                                            "ParentDeviceType": "part",
                                            "SizeBytes": "548682072064",
                                            "Mountpoint": "/var/lib/docker",
                                        },
                                        "name": "/dev/mapper/VolBDSCStore-thinpool",
                                    },
                                    {
                                        "info": {
                                            "IsDisk": False,
                                            "Name": "/dev/nvme0n1p1",
                                            "IsReadOnly": False,
                                            "IsRemovable": False,
                                            "IsRotational": False,
                                            "DeviceType": "part",
                                            "HasFilesystem": True,
                                            "IsLogicalVolume": False,
                                            "IsPartition": True,
                                            "ParentName": "/dev/nvme0n1",
                                            "ParentDeviceType": "disk",
                                            "SizeBytes": "429495664128",
                                            "Mountpoint": "/",
                                        },
                                        "name": "/dev/nvme0n1p1",
                                    },
                                    {
                                        "info": {
                                            "IsDisk": True,
                                            "Name": "/dev/nvme1n1",
                                            "IsReadOnly": False,
                                            "IsRemovable": False,
                                            "IsRotational": False,
                                            "DeviceType": "disk",
                                            "HasFilesystem": False,
                                            "IsLogicalVolume": False,
                                            "IsPartition": False,
                                            "ParentName": "",
                                            "ParentDeviceType": "",
                                            "SizeBytes": "549755813888",
                                            "Mountpoint": "",
                                        },
                                        "name": "/dev/nvme1n1",
                                    },
                                ],
                                "gpu": {"gpu_count": 0},
                                "swap": {"swap_total": 0},
                                "memory": {"mem_total": 66547093504},
                                "mountpoint": [],
                                "cpu": {
                                    "cpu_logical_cores": 16,
                                    "cpu_count": 8,
                                    "cpu_physical_cores": 8,
                                    "cpu_sockets": 1,
                                },
                            },
                            "approved_worker_pubkey": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQDk5SHYy1gwjEtERZNoIDmjgbomLa8psuM5q7+a0LvD3NsPDuvkQ2E2e32QAV5gCHy+yKteADIVK9BxonlQPxe/ArMzbHQjsfFaD+ek0oCr2xCs91wVTrK0LV/kTWLhgaVHDSJAw5U/36dnedMmmarxaV7GtzFVE2uzlIUl1Dr3Zvc7rZCBgLgB3cysVVzfBA9V/L1COl05GTC8jkRErASFX/F/93WbifNyF2+9Nnnnle/RX1Nc/GXaypjSLzGWIXFiuoztUP+DT4k/20izmMqEAF40LNHKpq6bkcjMU8S21boHjo7CtugIkhjwxp+k7dW0ls+SnvKx+3gi9sIFsTZxK7tBYd1dhPzHpl+G57N9YeusUXM4KAi1RGHR7Y1utWFUObfQi0Eh80b3T19gzHVS6AA7MCSbCOzxnWjgVZCMsckCm1YRiTsJ2vx1tcDsoEkl0sqivsF884ahWDZK/UBbx5ZJajrg56Mg27avsGXRc74U1OGRL1smF80OdBBeZ987+cqSqtp7lYDyfARh5CimE62Lljs7INWSQOWMx6ve4gSs/j+dj2e9H7YHXZKR3JvYzuf0J05u5i0jz55Kg2scmHsPgJsg6nMfAImPHtOW+c19olAhT5F5pbCBl9bHcdDgWXFZjjX9uZJCyW0ppLMXgke2qJxjPonG4n1oD49K3Q== server\n",
                            "propinfo": {
                                "bds_storage_apollo": "false",
                                "bds_storage_thinpooldev": "/dev/mapper/VolBDSCStore-thinpool",
                                "bds_network_publicinterface": "ens5",
                            },
                            "status_info": "",
                            "schedule": True,
                            "hacapable": True,
                            "tags": [],
                            "hdfs_disks_configured": "/dev/nvme2n1",
                            "container_disks_configured": "/dev/nvme1n1",
                            "hdfs_commission": True,
                            "hdfs_commission_status": "Normal",
                        },
                        {
                            "_links": {"self": {"href": "/api/v1/workers/2"}},
                            "ip": "10.1.0.196",
                            "purpose": "proxy",
                            "state": "installed",
                            "hostname": "ip-10-1-0-196.us-west-2.compute.internal",
                            "sysinfo": {
                                "network": [
                                    {
                                        "info": {
                                            "IsVirtual": False,
                                            "IpAddr": {
                                                "dynamic": "10.1.0.196/24"
                                            },
                                            "Mtu": 9001,
                                            "State": "up",
                                            "Carrier": True,
                                            "HwAddr": "02:51:ae:fd:6f:a8",
                                            "Speed": "UNKNOWN",
                                        },
                                        "name": "ens5",
                                    },
                                    {
                                        "info": {
                                            "IsVirtual": True,
                                            "IpAddr": {},
                                            "Mtu": 1500,
                                            "State": "UNKNOWN",
                                            "Carrier": "UNKNOWN",
                                            "HwAddr": "e6:5a:3f:7b:9f:41",
                                            "Speed": "UNKNOWN",
                                        },
                                        "name": "bds-flood",
                                    },
                                ],
                                "keys": {
                                    "reported_worker_public_key": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQCymOWzCBMjejpFgvrIcX3xr2XAoUtl5QIVUKYiq2lBS0NPYAJhe4wkp0NGLs640MuacHDfsizfA4iJspRElk7aZ9S6dYv4EeKsKAPGi0STobrFYy4J+Nxc8quXrJhq0Psj4N2YE2Y2NoDZI35tvLeQMgTAtRYgPeSri+shBba/wyHZan4ftR/mdOu1oNJAfXXQ0DIoUmkSqxgxGA2OwVHEh4eGw5j6idkN3HmdC911+3d7KePKKbqdsbV1tRA1rMcvCL94AykmSA8oqdecn3e3QD7OBiEWS2V10AlcvU0YWkG1Bp0TTKtmoXsU0wMegoAvETKcxzf1n+Y/aZ6/ZxzHODayAQu6R5IWDDtgcsGT4V8KCAP1EEJ47fsFFBoc+M21jAKLJPDuHSAANJUY1Z6+/RL5x1U5FmaLnbiL7QI3a7WJRtqPO9xZglW5mroolsjtdWR/gs6daNk/7W7Fy5MuPKmOE1nf1RAfCUh2WYcCInuQ5jnoXoz+5XndCBuJHu3Cy/ZpAF2PtSoZ52aRMnKD9wIb6flqVnVNViU/mf4Bn3r0l03PJMBoEodXGvezSM1jaQnyv8MwXsDk3NTqRA+AltCPFEu7rChz5B/XHcX7SLZK9/zNmi7jmHI5Hj22kQNTHP9FXf6Awvm2+k0TTk7HpGfNurW1igV090Imi23FGw== server\n"
                                },
                                "storage": [
                                    {
                                        "info": {
                                            "IsDisk": False,
                                            "Name": "/dev/nvme0n1p1",
                                            "IsReadOnly": False,
                                            "IsRemovable": False,
                                            "IsRotational": False,
                                            "DeviceType": "part",
                                            "HasFilesystem": True,
                                            "IsLogicalVolume": False,
                                            "IsPartition": False,
                                            "ParentName": "/dev/nvme0n1",
                                            "ParentDeviceType": "disk",
                                            "SizeBytes": "429495664128",
                                            "Mountpoint": "/",
                                        },
                                        "name": "/dev/nvme0n1p1",
                                    }
                                ],
                                "gpu": {"gpu_count": 0},
                                "swap": {"swap_total": 0},
                                "memory": {"mem_total": 66547085312},
                                "mountpoint": [],
                                "cpu": {
                                    "cpu_logical_cores": 16,
                                    "cpu_count": 8,
                                    "cpu_physical_cores": 8,
                                    "cpu_sockets": 1,
                                },
                            },
                            "approved_worker_pubkey": [],
                            "propinfo": {
                                "bds_storage_apollo": "false",
                                "bds_network_publicinterface": "ens5",
                            },
                            "status_info": "",
                            "schedule": False,
                            "hacapable": True,
                            "tags": [],
                            "proxy_nodes_hostname": "ip-10-1-0-196.us-west-2.compute.internal",
                        },
                    ]
                },
            },
            status_code=200,
            headers={},
        ),
    )


# purpose=controller (not proxy)
def mockApiGetWorkers97():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers/97",
        response=MockResponse(
            json_data={
                "hacapable": True,
                "propinfo": {
                    "bds_storage_apollo": "false",
                    "bds_network_publicinterface": "ens5",
                },
                "approved_worker_pubkey": ["test pub key"],
                "schedule": False,
                "ip": "10.1.0.37",
                "proxy_nodes_hostname": "ec2-35-165-137-87.us-west-2.compute.amazonaws.com",
                "hostname": "ip-10-1-0-37.us-west-2.compute.internal",
                "state": "installed",
                "_links": {"self": {"href": "/api/v1/workers/97"}},
                "purpose": "controller",
                "status_info": "test status info",
                "sysinfo": "test sysinfo",
                "tags": ["test tags"],
            },
            status_code=200,
            headers={},
        ),
    )


# worker with minimum sysinfo for testing
def mockApiGetWorkers98():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers/98",
        response=MockResponse(
            json_data={
                "hacapable": True,
                "propinfo": {
                    "bds_storage_apollo": "false",
                    "bds_network_publicinterface": "ens5",
                },
                "approved_worker_pubkey": ["test pub key"],
                "schedule": False,
                "ip": "10.1.0.37",
                "proxy_nodes_hostname": "ec2-35-165-137-87.us-west-2.compute.amazonaws.com",
                "hostname": "ip-10-1-0-37.us-west-2.compute.internal",
                "state": "installed",
                "_links": {"self": {"href": "/api/v1/workers/98"}},
                "purpose": "proxy",
                "status_info": "test status info",
                "sysinfo": "test sysinfo",
                "tags": ["test tags"],
            },
            status_code=200,
            headers={},
        ),
    )


def mockApiGetWorkers99():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers/99",
        response=MockResponse(
            json_data={
                "hacapable": True,
                "propinfo": {
                    "bds_storage_apollo": "false",
                    "bds_network_publicinterface": "ens5",
                },
                "approved_worker_pubkey": ["test pub key"],
                "schedule": False,
                "ip": "10.1.0.37",
                "proxy_nodes_hostname": "ec2-35-165-137-87.us-west-2.compute.amazonaws.com",
                "hostname": "ip-10-1-0-37.us-west-2.compute.internal",
                "state": "installed",
                "_links": {"self": {"href": "/api/v1/workers/99"}},
                "purpose": "proxy",
                "status_info": "test status info",
                "sysinfo": {
                    "keys": {
                        "reported_worker_public_key": (
                            "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQDKonff"
                            "u4vtTSINNpBwvLd367941fhPyEuVfh7KrohdIUSVEh/p"
                            "X8FDAO9fi9pH979AzdDVWeUclTmktm63vQ39TVIJQ+rq"
                            "doZUhtH8rSYFoTFzxUQxONviNJJGTiYYMo4kJsLO1Hk/"
                            "b9Lz8sxUJWD+e5r2UTM5cDSYT3wBHUCDr/MXAxNC9FAg"
                            "kpuME5utC1dd1aHj2zgLUP61REjnhy1zVVJnbh/T/y3p"
                            "8Z5z0ubAQy7pYaMTuWgdVMH6kA/RWzOB2JRj8vFKYp9f"
                            "ysFe7L/nj+C2LkDr4dmMLNL9ffTvpnMOj5qPgAO8bay5"
                            "hAgVykUaRInLjuL7p5/nFATm9uI4A2a28m4HO9csywNX"
                            "pm5TBDWPDxW7Wh7Sdkx0xHwZenXXy/em+4Q4Fk4Oc6Yw"
                            "YcKOJVsst0qGeCFkhLjzvFHu2ceYf5Q1gg5FlBiX+LsW"
                            "ngjArsd0sdh+3piH/xFuHdubqHfOFpOlZsQsMX5V/LUA"
                            "71Wqv/cxMsoD5jybQOUS8o34JjkCZlavuJcIeU4hWlWE"
                            "liZU5SmppuNkHdosXup20/TyBgg0qYlzc+FKZ/8vlQSj"
                            "T5WgCNffPgXR94KPF1817RW1YSbR+1oiNg6FXgQrKM/1"
                            "DiqyQ5D8DjhZWgg33hJ7K/fKCL3qPyWCJEMQ64iLQ4Qt"
                            "SeU46l+aO490A89u6w== server\n"
                        )
                    },
                    "storage": [
                        {
                            "info": {
                                "IsLogicalVolume": False,
                                "IsDisk": False,
                                "Name": "/dev/nvme0n1p1",
                                "SizeBytes": "429495664128",
                                "IsReadOnly": False,
                                "ParentName": "/dev/nvme0n1",
                                "IsRemovable": False,
                                "IsRotational": False,
                                "ParentDeviceType": "disk",
                                "IsPartition": True,
                                "DeviceType": "part",
                                "HasFilesystem": True,
                                "Mountpoint": "/",
                            },
                            "name": "/dev/nvme0n1p1",
                        }
                    ],
                    "swap": {"swap_total": 0},
                    "memory": {"mem_total": 65842458624},
                    "gp": {"gpu_count": 0},
                    "cp": {
                        "cpu_logical_cores": 16,
                        "cpu_count": 8,
                        "cpu_physical_cores": 8,
                        "cpu_sockets": 1,
                    },
                    "mountpoint": [],
                },
                "tags": ["test tags"],
            },
            status_code=200,
            headers={},
        ),
    )


def mockApiGetWorkers100():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers/100",
        response=MockResponse(
            json_data={
                "hacapable": True,
                "propinfo": {
                    "bds_storage_apollo": "false",
                    "bds_network_publicinterface": "ens5",
                },
                "approved_worker_pubkey": ["test pub key"],
                "schedule": False,
                "ip": "10.1.0.37",
                "proxy_nodes_hostname": (
                    "ec2-35-165-137-87.us-west-2.compute.amazonaws.com"
                ),
                "hostname": "ip-10-1-0-37.us-west-2.compute.internal",
                "state": "installed",
                "_links": {"self": {"href": "/api/v1/workers/100"}},
                "purpose": "controller",
                "status_info": "",
                "sysinfo": {
                    "network": [
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "7e:d0:19:00:a1:c0",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-6-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "52:c4:7b:93:f2:0a",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-2-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "b2:e6:2b:c5:7a:d4",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-0-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "UNKNOWN",
                                "Carrier": "UNKNOWN",
                                "HwAddr": "c6:b3:cd:1b:7d:44",
                                "Speed": "UNKNOWN",
                            },
                            "name": "bds-flood",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "5a:16:20:0c:d7:f1",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-1-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "c6:bf:e3:af:82:2f",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-1-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "0e:d9:23:62:96:94",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-0-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "e2:86:7e:62:c0:3c",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-2-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "8e:92:dc:b9:b7:d7",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-6-l",
                        },
                        {
                            "info": {
                                "IsVirtual": False,
                                "IpAddr": {"dynamic": "10.1.0.37/24"},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "02:72:98:8e:3b:86",
                                "Speed": "UNKNOWN",
                            },
                            "name": "ens5",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "2a:4a:c9:c6:d0:28",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-4-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "7a:22:39:fc:11:7e",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-5-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "9a:2f:6c:e6:74:30",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-3-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "7e:ff:e4:5b:56:0d",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-7-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "4a:f7:7e:95:14:2d",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-7-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "36:5f:82:61:bf:5c",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-3-r",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "6e:86:7d:52:94:5d",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-5-l",
                        },
                        {
                            "info": {
                                "IsVirtual": True,
                                "IpAddr": {},
                                "Mt": 9001,
                                "State": "up",
                                "Carrier": True,
                                "HwAddr": "e6:24:0f:54:99:a9",
                                "Speed": "10000",
                            },
                            "name": "bds-flood-4-l",
                        },
                    ],
                    "keys": {
                        "reported_worker_public_key": (
                            "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQDKonff"
                            "u4vtTSINNpBwvLd367941fhPyEuVfh7KrohdIUSVEh/p"
                            "X8FDAO9fi9pH979AzdDVWeUclTmktm63vQ39TVIJQ+rq"
                            "oZUhtH8rSYFoTFzxUQxONviNJJGTiYYMo4kJsLO1Hk/b"
                            "9Lz8sxUJWD+e5r2UTM5cDSYT3wBHUCDr/MXAxNC9FAgk"
                            "puME5utC1dd1aHj2zgLUP61REjnhy1zVVJnbh/T/y3p8"
                            "Z5z0ubAQy7pYaMTuWgdVMH6kA/RWzOB2JRj8vFKYp9fy"
                            "sFe7L/nj+C2LkDr4dmMLNL9ffTvpnMOj5qPgAO8bay5h"
                            "AgVykUaRInLjuL7p5/nFATm9uI4A2a28m4HO9csywNXp"
                            "m5TBDWPDxW7Wh7Sdkx0xHwZenXXy/em+4Q4Fk4Oc6YwY"
                            "cKOJVsst0qGeCFkhLjzvFHu2ceYf5Q1gg5FlBiX+LsWn"
                            "gjArsd0sdh+3piH/xFuHdubqHfOFpOlZsQsMX5V/LUA7"
                            "1Wqv/cxMsoD5jybQOUS8o34JjkCZlavuJcIeU4hWlWEl"
                            "iZU5SmppuNkHdosXup20/TyBgg0qYlzc+FKZ/8vlQSjT"
                            "5WgCNffPgXR94KPF1817RW1YSbR+1oiNg6FXgQrKM/1D"
                            "iqyQ5D8DjhZWgg33hJ7K/fKCL3qPyWCJEMQ64iLQ4QtS"
                            "eU46l+aO490A89u6w== server\n"
                        )
                    },
                    "storage": [
                        {
                            "info": {
                                "IsLogicalVolume": False,
                                "IsDisk": False,
                                "Name": "/dev/nvme0n1p1",
                                "SizeBytes": "429495664128",
                                "IsReadOnly": False,
                                "ParentName": "/dev/nvme0n1",
                                "IsRemovable": False,
                                "IsRotational": False,
                                "ParentDeviceType": "disk",
                                "IsPartition": True,
                                "DeviceType": "part",
                                "HasFilesystem": True,
                                "Mountpoint": "/",
                            },
                            "name": "/dev/nvme0n1p1",
                        }
                    ],
                    "swap": {"swap_total": 0},
                    "memory": {"mem_total": 65842458624},
                    "gp": {"gpu_count": 0},
                    "cp": {
                        "cpu_logical_cores": 16,
                        "cpu_count": 8,
                        "cpu_physical_cores": 8,
                        "cpu_sockets": 1,
                    },
                    "mountpoint": [],
                },
                "tags": [],
            },
            status_code=200,
            headers={},
        ),
    )


# worker that doesn't exist
def mockApiGetWorkers999():
    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/workers/999",
        response=MockResponse(
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )


def mockApiPostSetup():

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v1/workers/",
        response=MockResponse(
            json_data={
                "hacapable": True,
                "propinfo": {
                    "bds_storage_apollo": "false",
                    "bds_network_publicinterface": "ens5",
                },
                "approved_worker_pubkey": ["test pub key"],
                "schedule": False,
                "ip": "10.1.0.37",
                "hostname": "ip-10-1-0-37.us-west-2.compute.internal",
                "state": "installed",
                "_links": {"self": {"href": "/api/v1/workers/99"}},
                "purpose": "proxy",
                "status_info": "test status info",
                "sysinfo": "test sysinfo",
                "tags": ["test tags"],
            },
            status_code=200,
            headers={"Location": "/api/v1/workers/123"},
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost",
        response=MockResponse(
            json_data={
                "_embedded": {
                    "k8shosts": [
                        {
                            "status": "unlicensed",
                            "propinfo": {
                                "bds_storage_apollo": "false",
                                "bds_network_publicinterface": "ens5",
                            },
                            "approved_worker_pubkey": [],
                            "tags": [],
                            "hostname": (
                                "ip-10-1-0-238.eu-west-2.compute.internal"
                            ),
                            "ipaddr": "10.1.0.238",
                            "setup_log": (
                                "/var/log/bluedata/install/"
                                "k8shost_setup_10.1.0.238-"
                                "2020-4-26-18-41-16"
                            ),
                            "_links": {
                                "self": {"href": "/api/v2/worker/k8shost/4"}
                            },
                            "sysinfo": {
                                "network": [],
                                "keys": {
                                    "reported_worker_public_key": (
                                        "ssh-rsa ...== server\n"
                                    )
                                },
                                "storage": [],
                                "swap": {"swap_total": 0},
                                "memory": {"mem_total": 65842503680},
                                "gpu": {"gpu_count": 0},
                                "cpu": {
                                    "cpu_logical_cores": 16,
                                    "cpu_count": 8,
                                    "cpu_physical_cores": 8,
                                    "cpu_sockets": 1,
                                },
                                "mountpoint": [],
                            },
                        },
                        {
                            "status": "bundle",
                            "approved_worker_pubkey": [],
                            "tags": [],
                            "hostname": "",
                            "ipaddr": "10.1.0.186",
                            "setup_log": (
                                "/var/log/bluedata/install/"
                                "k8shost_setup_10.1.0.186-"
                                "2020-4-26-18-49-10"
                            ),
                            "_links": {
                                "self": {"href": "/api/v2/worker/k8shost/5"}
                            },
                        },
                    ]
                }
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost/5",
        response=MockResponse(
            json_data={
                "status": "bundle",
                "approved_worker_pubkey": [],
                "tags": [],
                "hostname": "",
                "ipaddr": "10.1.0.186",
                "setup_log": (
                    "/var/log/bluedata/install/"
                    "k8shost_setup_10.1.0.186-"
                    "2020-4-26-18-49-10"
                ),
                "_links": {"self": {"href": "/api/v2/worker/k8shost/5"}},
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost/5?setup_log=true",
        response=MockResponse(
            json_data={
                "status": "bundle",
                "approved_worker_pubkey": [],
                "tags": [],
                "hostname": "",
                "ipaddr": "10.1.0.186",
                "setup_log": (
                    "/var/log/bluedata/install/"
                    "k8shost_setup_10.1.0.186-"
                    "2020-4-26-18-49-10"
                ),
                "_links": {"self": {"href": "/api/v2/worker/k8shost/5"}},
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost/8",
        response=MockResponse(
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost/5",
        response=MockResponse(json_data={}, status_code=204, headers={}),
    )

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v2/worker/k8shost/",
        response=MockResponse(
            json_data={},
            status_code=201,
            headers={"location": "/new/cluster/id"},
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import tempfile

from mock import patch

from hpecp import APIItemNotFoundException
from hpecp.exceptions import APIItemConflictException
from hpecp.gateway import GatewayController, GatewayStatus

from .base import BaseTestCase, MockResponse, get_client
from .gateway_mock_api_responses import mockApiGetSetup, mockApiPostSetup


def session_mock_response():
    return MockResponse(
        json_data={},
        status_code=200,
        headers={
            "location": "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
        },
    )


class TestGatewayList(BaseTestCase):
    def setUp(self):
        mockApiGetSetup()
        # mockApiPostSetup()
        super(TestGatewayList, self).setUp()

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_list(self, mock_get, mock_post):

        # calls POST https://127.0.0.1:8080/api/v1/login
        # calls GET https://127.0.0.1:8080/api/v1/workers

        try:
            get_client().gateway.list()
        except Exception as e:
            # Unexpected exception
            self.fail(e)


class TestGatewayGet(BaseTestCase):
    def setUp(self):
        mockApiGetSetup()
        mockApiPostSetup()
        super(TestGatewayGet, self).setUp()

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_gateway_assertions(self, mock_get, mock_post):

        with self.assertRaisesRegexp(
            AssertionError,
            "'id' must be provided and must be a str",
        ):
            get_client().gateway.get(123)

        with self.assertRaisesRegexp(
            AssertionError, "'id' does not start with '/api/v1/workers'"
        ):
            get_client().gateway.get("garbage")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_gateway(self, mock_get, mock_post):

        gateway = get_client().gateway.get("/api/v1/workers/99")

        self.assertEqual(gateway.id, "/api/v1/workers/99")
        self.assertEqual(gateway.hacapable, True)
        self.assertEqual(
            gateway.propinfo,
            {
                "bds_network_publicinterface": "ens5",
                "bds_storage_apollo": "false",
            },
        )
        self.assertEqual(gateway.approved_worker_pubkey, ["test pub key"])
        self.assertEqual(gateway.schedule, False)
        self.assertEqual(gateway.ip, "10.1.0.37")
        self.assertEqual(
            gateway.proxy_nodes_hostname,
            "ec2-35-165-137-87.us-west-2.compute.amazonaws.com",
        )
        self.assertEqual(
            gateway.hostname, "ip-10-1-0-37.us-west-2.compute.internal"
        )
        self.assertEqual(gateway.purpose, "proxy")
        self.assertEqual(gateway.status_info, "test status info")
        self.assertEqual(gateway.tags, ["test tags"])

        # /api/v1/workers/97 has "'purpose': 'controller'" so it
        #  isn't a gateway
        with self.assertRaisesRegexp(
            APIItemNotFoundException,
            "'gateway not found with id: /api/v1/workers/97'",
        ):
            get_client().gateway.get("/api/v1/workers/97")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_gateway_sysinfo(self, mock_get, mock_post):

        gateway = get_client().gateway.get("/api/v1/workers/98")

        self.assertEqual(gateway.sysinfo, "test sysinfo")
        self.assertEqual(
            gateway.proxy_nodes_hostname,
            "ec2-35-165-137-87.us-west-2.compute.amazonaws.com",
        )


class TestCreateGateway(BaseTestCase):
    def setUp(self):
        mockApiGetSetup()
        mockApiPostSetup()
        super(TestCreateGateway, self).setUp()

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create_with_ssh_key_assertions(self, mock_post):

        with self.assertRaisesRegexp(
            AssertionError, "'ip' must be provided and must be a string"
        ):
            get_client().gateway.create_with_ssh_key(
                ip=123,
                proxy_node_hostname="my.host.name",
                ssh_key_data="pem encoded key data",
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'proxy_node_hostname' must be provided and must be a string",
        ):
            get_client().gateway.create_with_ssh_key(
                ip="127.0.0.1",
                proxy_node_hostname=1234,
                ssh_key_data="pem encoded key data",
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'ssh_key_data' must be provided and must be a string",
        ):
            get_client().gateway.create_with_ssh_key(
                ip="127.0.0.1",
                proxy_node_hostname="abc",
                ssh_key_data=1234,
            )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create_with_ssh_key_returns_id(self, mock_post):

        get_client().gateway.create_with_ssh_key(
            ip="127.0.0.1",
            proxy_node_hostname="my.host.name",
            ssh_key_data="pem encoded key data",
        )


class TestWaitForGatewayStatus(BaseTestCase):
    def setUp(self):
        mockApiGetSetup()
        mockApiPostSetup()
        super(TestWaitForGatewayStatus, self).setUp()

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_assertions(self, mock_get, mock_post):

        # FIXME speed these tests up

        with self.assertRaisesRegexp(
            AssertionError, "'id' must be provided and must be a str"
        ):
            get_client().gateway.wait_for_state(
                gateway_id=1, timeout_secs=1, state=[GatewayStatus.ready]
            )

        # pylint: disable=anomalous-backslash-in-string
        with self.assertRaisesRegexp(
            AssertionError, "'id' does not start with '/api/v1/workers'"
        ):
            get_client().gateway.wait_for_state(
                gateway_id="garbage",
                timeout_secs=1,
                state=[GatewayStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError, "'timeout_secs' must be an int"
        ):
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/123",
                timeout_secs="blah",
                state=[GatewayStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError, "'timeout_secs' must be >= 0"
        ):
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/123",
                timeout_secs=-1,
                state=[GatewayStatus.ready],
            )

        with self.assertRaisesRegexp(
            AssertionError,
            "'status' item '0' is not of type <enum 'GatewayStatus'>",
        ):
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/123", timeout_secs=1, state=["abc"]
            )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway(self, mock_get, mock_post):

        self.assertTrue(
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/98",
                timeout_secs=1,
                state=[GatewayStatus.installed],
            )
        )

        self.assertFalse(
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/98",
                timeout_secs=1,
                state=[GatewayStatus.deleting],
            )
        )

        self.assertTrue(
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/98",
                timeout_secs=1,
                state=[GatewayStatus.installed, GatewayStatus.deleting],
            )
        )

        self.assertFalse(
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/98",
                timeout_secs=1,
                state=[GatewayStatus.error, GatewayStatus.deleting],
            )
        )

        # Get the status of a Cluster ID that doesn't exist
        with self.assertRaises(APIItemNotFoundException):
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/999",
                timeout_secs=1,
                state=[GatewayStatus.installed],
            )

        # Get the status of a Cluster ID that doesn't
        # exist - without providing a status
        self.assertTrue(
            get_client().gateway.wait_for_state(
                gateway_id="/api/v1/workers/999", timeout_secs=1, state=[]
            )
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli(self, mock_get, mock_post):

        hpecp = self.cli.CLI()
        hpecp.gateway.wait_for_state(
            id="/api/v1/workers/98",
            timeout_secs=1,
            states=[GatewayStatus.installed.name],
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli_fail_to_reach_state(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.gateway.wait_for_state(
                id="/api/v1/workers/98",
                timeout_secs=1,
                states=[GatewayStatus.deleting.name],
            )
        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = "Failed to reach state(s) ['deleting'] in 1s"

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli_multiple_states(
        self, mock_get, mock_post
    ):

        hpecp = self.cli.CLI()
        hpecp.gateway.wait_for_state(
            id="/api/v1/workers/98",
            timeout_secs=1,
            states=[
                GatewayStatus.installed.name,
                GatewayStatus.deleting.name,
            ],
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli_fail_to_reach_state_with_multiple_states(
        self, mock_get, mock_post
    ):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.gateway.wait_for_state(
                id="/api/v1/workers/123",
                timeout_secs=1,
                states=[GatewayStatus.error.name, GatewayStatus.deleting.name],
            )
        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = (
            "Failed to reach state(s) ['error', 'deleting'] in 1s"
        )

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli_gateway_id_does_not_exist(
        self, mock_get, mock_post
    ):

        # Get the status of a ID that doesn't exist
        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.gateway.wait_for_state(
                id="/api/v1/workers/999",
                timeout_secs=1,
                states=[GatewayStatus.installed.name],
            )
        self.assertEqual(cm.exception.code, 1)

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_status_gateway_cli_gateway_id_does_not_exist_and_no_status(
        self, mock_get, mock_post
    ):

        # Get the status of a Cluster ID that doesn't
        # exist - without providing a status
        try:
            hpecp = self.cli.CLI()
            hpecp.gateway.wait_for_state(
                id="/api/v1/workers/999", timeout_secs=1, states=[]
            )
        except SystemExit:
            self.fail("Should not raise a SystemExit")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_wait_for_delete_gateway_cli_gateway_id_does_not_exist_and_no_status(
        self, mock_get, mock_post
    ):

        try:
            hpecp = self.cli.CLI()
            hpecp.gateway.wait_for_delete(
                id="/api/v1/workers/999", timeout_secs=1
            )
        except SystemExit:
            self.fail("Should not raise a SystemExit")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_states(self, mock_post):

        # TODO move me - I don't really belong in the
        #      wait_for_state test cases

        hpecp = self.cli.CLI()
        hpecp.gateway.states()

        stdout = self.out.getvalue().strip()

        expected_stdout = (
            "['bundle', 'installing', 'installed', 'ready', "
            "'unlicensed', 'configuring', 'configured', 'error', "
            "'sysinfo', 'unconfiguring', 'deleting', "
            "'storage_pending', 'storage_configuring', "
            "'storage_error', 'decommission_in_progress', "
            "'delete_in_progress']"
        )

        self.assertEqual(stdout, expected_stdout)


class TestDeleteGateway(BaseTestCase):
    # pylint: disable=no-method-argument
    def mocked_requests_get(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/workers/123":
            return MockResponse(
                json_data={
                    "_links": {"self": {"href": "/api/v1/workers/123"}},
                    "purpose": "proxy",
                },
                status_code=200,
                headers={},
            )
        if args[0] == "https://127.0.0.1:8080/api/v1/workers/999":
            return MockResponse(
                text_data="Not found.",
                json_data={},
                status_code=404,
                raise_for_status_flag=True,
                headers={},
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    # pylint: disable=no-method-argument
    def mocked_requests_delete(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/workers/999":
            return MockResponse(
                text_data="Not found.",
                json_data={},
                status_code=404,
                raise_for_status_flag=True,
                headers={},
            )
        if args[0] == "https://127.0.0.1:8080/api/v1/workers/123":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={},
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    def mocked_requests_post(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    # delete() does a get() request to check the worker has 'purpose':'proxy'
    @patch("requests.get", side_effect=mocked_requests_get)
    @patch("requests.delete", side_effect=mocked_requests_delete)
    @patch("requests.post", side_effect=mocked_requests_post)
    def test_delete_gateway(self, mock_get, mock_post, mock_delete):

        with self.assertRaisesRegexp(
            AssertionError, "'id' does not start with '/api/v1/workers'"
        ):
            get_client().gateway.delete(id="garbage")

        with self.assertRaises(APIItemNotFoundException):
            get_client().gateway.delete(id="/api/v1/workers/999")

        get_client().gateway.delete(id="/api/v1/workers/123")


class TestCliCreate(BaseTestCase):
    def test_key_or_keycontent_provided(
        self,
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.gateway.create_with_ssh_key(
                ip="127.0.0.1", proxy_node_hostname="somehost"
            )

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = "Either ssh_key or ssh_key_file must be provided"

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )

    def test_key_and_keycontent_provided(
        self,
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.gateway.create_with_ssh_key(
                ip="127.0.0.1",
                proxy_node_hostname="somehost",
                ssh_key="foobar",
                ssh_key_file="foobar",
            )

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = "Either ssh_key or ssh_key_file must be provided"

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )

    def mocked_requests_post(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return session_mock_response()
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_post)
    @patch("hpecp.gateway")
    def test_with_only_ssh_key_content_provided(self, mock_post, mock_gateway):

        """Test that the ssh key content provided by the 'ssh_key' parameter
        is passed to the library method 'create_with_ssh_key()'.
        """

        with patch.object(
            GatewayController,
            "create_with_ssh_key",
            return_value="/api/v1/workers/1",
        ) as mock_create_with_ssh_key:
            try:
                hpecp = self.cli.CLI()
                hpecp.gateway.create_with_ssh_key(
                    ip="127.0.0.1",
                    proxy_node_hostname="somehost",
                    ssh_key="test_ssh_key",
                )
            except Exception:
                self.fail("Unexpected exception.")

        mock_create_with_ssh_key.assert_called_once_with(
            ip="127.0.0.1",
            proxy_node_hostname="somehost",
            ssh_key_data="test_ssh_key",
            tags=[],
        )

        stdout = self.out.getvalue().strip()

        self.assertEqual(stdout, "/api/v1/workers/1")

    @patch("requests.post", side_effect=mocked_requests_post)
    @patch("hpecp.gateway")
    def test_with_only_ssh_key_content_provided_raises_assertion_error(
        self, mock_post, mock_gateway
    ):

        with patch.object(
            GatewayController,
            "create_with_ssh_key",
            side_effect=AssertionError("TEST_ASSERTION"),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp = self.cli.CLI()
                hpecp.gateway.create_with_ssh_key(
                    ip="127.0.0.1",
                    proxy_node_hostname="somehost",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = "TEST_ASSERTION"

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch("requests.post", side_effect=mocked_requests_post)
    @patch("hpecp.gateway")
    def test_with_only_ssh_key_content_provided_raises_conflict_exception(
        self, mock_post, mock_gateway
    ):

        with patch.object(
            GatewayController,
            "create_with_ssh_key",
            side_effect=APIItemConflictException(
                message="APIItemConflictException.message",
                request_method="METHOD",
                request_url="URL",
            ),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp = self.cli.CLI()
                hpecp.gateway.create_with_ssh_key(
                    ip="127.0.0.1",
                    proxy_node_hostname="somehost",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = "APIItemConflictException.message"

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch("requests.post", side_effect=mocked_requests_post)
    @patch("hpecp.gateway")
    def test_with_only_ssh_key_content_provided_raises_general_exception(
        self, mock_post, mock_gateway
    ):

        with patch.object(
            GatewayController,
            "create_with_ssh_key",
            side_effect=Exception("TEST_EXCEPTION"),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp = self.cli.CLI()
                hpecp.gateway.create_with_ssh_key(
                    ip="127.0.0.1",
                    proxy_node_hostname="somehost",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch("requests.post", side_effect=mocked_requests_post)
    @patch("hpecp.gateway")
    def test_with_only_ssh_key_file_provided(self, mock_post, mock_gateway):

        ssh_key_file = tempfile.NamedTemporaryFile(delete=True, mode="w")
        ssh_key_file.write("test_ssh_key_file_data")
        ssh_key_file.flush()

        with patch.object(
            GatewayController,
            "create_with_ssh_key",
            return_value="/api/v1/workers/1",
        ) as mock_create_with_ssh_key:
            try:
                hpecp = self.cli.CLI()
                hpecp.gateway.create_with_ssh_key(
                    ip="127.0.0.1",
                    proxy_node_hostname="somehost",
                    ssh_key_file=ssh_key_file.name,
                )
            except Exception as e:
                self.fail("Unexpected exception. {}".format(e))

        mock_create_with_ssh_key.assert_called_once_with(
            ip="127.0.0.1",
            proxy_node_hostname="somehost",
            ssh_key_data="test_ssh_key_file_data",
            tags=[],
        )

        stdout = self.out.getvalue().strip()

        self.assertEqual(stdout, "/api/v1/workers/1")

        ssh_key_file.close()


class TestCliDelete(BaseTestCase):
    def mocked_requests_post(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_post)
    # @patch("requests.del", side_effect=mocked_requests_delete)
    def test_delete_with_unknown_exception(self, mock_post):
        @patch("hpecp.base_resource.AbstractController")
        def delete(self, id):
            raise Exception()

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.gateway.delete("/api/v1/workers/1")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""  # we don't want error output going to stdout
        expected_stderr = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(stderr.endswith(expected_stderr))



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from hpecp.exceptions import APIItemNotFoundException

from .base import BaseTestCase, MockResponse


def mockApiSetup():
    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v1/catalog/99",
        response=MockResponse(json_data={}, status_code=204, headers=dict()),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/catalog/99",
        response=MockResponse(
            json_data={
                "_links": {
                    "self": {"href": "/api/v1/catalog/99"},
                    "feed": [
                        {
                            "href": (
                                "https://s3.amazonaws.com/bluedata-catalog/"
                                "bundles/catalog/external/docker/EPIC-5.0/"
                                "feeds/feed.json"
                            ),
                            "name": (
                                "BlueData EPIC-5.0 catalog feed for docker"
                            ),
                        }
                    ],
                },
                "id": "/api/v1/catalog/99",
                "distro_id": "bluedata/spark240juphub7xssl",
                "label": {
                    "name": "Spark240",
                    "description": (
                        "Spark240 multirole with Jupyter Notebook, Jupyterhub"
                        " with SSL and gateway node"
                    ),
                },
                "version": "2.8",
                "timestamp": 0,
                "isdebug": False,
                "osclass": ["centos"],
                "logo": {
                    "checksum": "1471eb59356066ed4a06130566764ea6",
                    "url": (
                        "http://10.1.0.53/catalog/logos/"
                        "bluedata-spark240juphub7xssl-2.8"
                    ),
                },
                "documentation": {
                    "checksum": "52f53f1b2845463b9e370d17fb80bea6",
                    "mimetype": "text/markdown",
                    "file": (
                        "/opt/bluedata/catalog/documentation/"
                        "bluedata-spark240juphub7xssl-2.8"
                    ),
                },
                "state": "initialized",
                "state_info": "",
            },
            status_code=200,
            headers=dict(),
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/catalog/100",
        response=MockResponse(
            json_data={
                "_links": {
                    "self": {"href": "/api/v1/catalog/100"},
                    "feed": [
                        {
                            "href": (
                                "https://s3.amazonaws.com/bluedata-catalog/"
                                "bundles/catalog/external/docker/EPIC-5.0/"
                                "feeds/feed.json"
                            ),
                            "name": (
                                "BlueData EPIC-5.0 catalog feed for docker"
                            ),
                        }
                    ],
                },
                "id": "/api/v1/catalog/100",
                "distro_id": "bluedata/spark240juphub7xssl",
                "label": {
                    "name": "Spark240",
                    "description": (
                        "Spark240 multirole with Jupyter Notebook, Jupyterhub"
                        " with SSL and gateway node"
                    ),
                },
                "version": "2.8",
                "timestamp": 0,
                "isdebug": False,
                "osclass": ["centos"],
                "logo": {
                    "checksum": "1471eb59356066ed4a06130566764ea6",
                    "url": (
                        "http://10.1.0.53/catalog/logos/"
                        "bluedata-spark240juphub7xssl-2.8"
                    ),
                },
                "documentation": {
                    "checksum": "52f53f1b2845463b9e370d17fb80bea6",
                    "mimetype": "text/markdown",
                    "file": (
                        "/opt/bluedata/catalog/documentation/"
                        "bluedata-spark240juphub7xssl-2.8"
                    ),
                },
                "state": "initialized",
                "state_info": "",
            },
            status_code=200,
            headers=dict(),
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/catalog/101",
        response=APIItemNotFoundException(
            message="catalog not found with id: " + "/api/v1/catalog/101",
            request_method="get",
            request_url="https://127.0.0.1:8080/api/v1/catalog/101",
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/catalog/102",
        response=MockResponse(
            json_data={"garbage"},
            status_code=200,
            headers=dict(),
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/catalog",
        response=MockResponse(
            json_data={
                "_links": {
                    "self": {"href": "/api/v1/catalog/"},
                    "feedlog": {"href": "/api/v1/catalog/feedlog"},
                    "feed": [
                        {
                            "href": "http://127.0.0.1:8080/api/v1/feed/local",
                            "name": "Feed generated from local bundles.",
                        },
                        {
                            "href": (
                                "https://s3.amazonaws.com/bluedata-catalog/bundles/"
                                "catalog/external/docker/EPIC-5.0/feeds/feed.json"
                            ),
                            "name": "BlueData EPIC-5.0 catalog feed for docker",
                        },
                    ],
                },
                "catalog_api_version": 6,
                "feeds_refresh_period_seconds": 86400,
                "feeds_read_counter": 5,
                "catalog_write_counter": 5,
                "_embedded": {
                    "independent_catalog_entries": [
                        {
                            "_links": {
                                "self": {"href": "/api/v1/catalog/29"},
                                "feed": [
                                    {
                                        "href": (
                                            "https://s3.amazonaws.com/bluedata-catalog/"
                                            "bundles/catalog/external/docker/EPIC-5.0/"
                                            "feeds/feed.json"
                                        ),
                                        "name": (
                                            "BlueData EPIC-5.0 catalog feed for docker"
                                        ),
                                    }
                                ],
                            },
                            "distro_id": "bluedata/spark240juphub7xssl",
                            "label": {
                                "name": "Spark240",
                                "description": (
                                    "Spark240 multirole with Jupyter Notebook, Jupyterhub"
                                    " with SSL and gateway node"
                                ),
                            },
                            "version": "2.8",
                            "timestamp": 0,
                            "isdebug": False,
                            "osclass": ["centos"],
                            "logo": {
                                "checksum": "1471eb59356066ed4a06130566764ea6",
                                "url": (
                                    "http://10.1.0.53/catalog/logos/"
                                    "bluedata-spark240juphub7xssl-2.8"
                                ),
                            },
                            "documentation": {
                                "checksum": "52f53f1b2845463b9e370d17fb80bea6",
                                "mimetype": "text/markdown",
                                "file": (
                                    "/opt/bluedata/catalog/documentation/"
                                    "bluedata-spark240juphub7xssl-2.8"
                                ),
                            },
                            "state": "initialized",
                            "state_info": "",
                        }
                    ]
                },
            },
            status_code=200,
            headers=dict(),
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v2/hpelicense",
        response=MockResponse(
            json_data={},
            status_code=201,
            headers={"location": "/api/v2/hpeclicense/1"},
        ),
    )

    BaseTestCase.registerHttpDeleteHandler(
        url="https://127.0.0.1:8080/api/v2/hpelicense/TEST_LICENSE_KEY/",
        response=MockResponse(
            json_data={},
            status_code=200,
            headers=dict(),
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/license",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v1/license"}},
                "state": "unlicensed",
                "uuid": "3c831f6e-f76f-410d-977c-ed13b0c817d1",
            },
            status_code=200,
            headers=dict(),
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v2/hpelicense",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v2/hpelicense"}},
                "Licenses": [
                    {
                        "Label": "The License",
                        "Feature": "HPE Machine Learning Ops",
                        "Capacity": 240,
                        "UnlimitedCapacity": False,
                        "Start": 1566864000000,
                        "StartDisplay": "2019-08-27T00:00:00Z",
                        "Expiration": 1609286399000,
                        "ExpirationDisplay": "2020-12-29T23:59:59Z",
                        "LicenseKey": "TEST_LICENSE_KEY",
                        "DeviceID": "1234 1234",
                        "Evaluation": False,
                    }
                ],
                "Summaries": [
                    {
                        "Label": "HPE Container Platform",
                        "UnlimitedCapacity": False,
                        "TotalCapacity": 240,
                        "UsedCapacity": 24,
                        "AvailableCapacity": 216,
                        "NextExpiration": 1609286399000,
                        "NextExpirationDisplay": "2020-12-29T23:59:59Z",
                        "LatestExpiration": 1609286399000,
                        "LatestExpirationDisplay": "2020-12-29T23:59:59Z",
                        "Valid": True,
                        "ValidationTime": 1594758782000,
                        "RevalidateTime": 1609286400000,
                    },
                    {
                        "Label": "HPE Machine Learning Ops",
                        "UnlimitedCapacity": False,
                        "TotalCapacity": 240,
                        "UsedCapacity": 0,
                        "AvailableCapacity": 240,
                        "NextExpiration": 1609286399000,
                        "NextExpirationDisplay": "2020-12-29T23:59:59Z",
                        "LatestExpiration": 1609286399000,
                        "LatestExpirationDisplay": "2020-12-29T23:59:59Z",
                        "Valid": True,
                        "ValidationTime": 1594758782000,
                        "RevalidateTime": 1609286400000,
                    },
                ],
                "Messages": [],
                "Valid": True,
                "Enabled": True,
                "ValidationTime": 1594758782000,
                "RevalidateTime": 1609286400000,
            },
            status_code=200,
            headers=dict(),
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/tenant/",
        response=MockResponse(
            json_data={
                "_embedded": {
                    "tenants": [
                        {
                            "status": "ready",
                            "tenant_type": "docker",
                            "features": {
                                "ml_project": False,
                                "kubernetes_access": False,
                            },
                            "persistent_supported": True,
                            "member_key_available": "all_admins",
                            "quota": {},
                            "cluster_isolation_supported": True,
                            "inusequota": {
                                "disk": 0,
                                "cores": 0,
                                "memory": 0,
                                "persistent": 0,
                                "gpus": 0,
                            },
                            "external_user_groups": [],
                            "gpu_usage_supported": True,
                            "_links": {"self": {"href": "/api/v1/tenant/1"}},
                            "filesystem_mount_supported": True,
                            "tenant_enforcements": [],
                            "label": {
                                "name": "Site Admin",
                                "description": (
                                    "Site Admin Tenant"
                                    " for BlueData clusters"
                                ),
                            },
                            "constraints_supported": False,
                            "tenant_storage_quota_supported": False,
                        },
                        {
                            "status": "ready",
                            "tenant_type": "docker",
                            "features": {
                                "ml_project": False,
                                "kubernetes_access": False,
                            },
                            "persistent_supported": True,
                            "member_key_available": "all_admins",
                            "quota": {},
                            "cluster_isolation_supported": True,
                            "inusequota": {
                                "disk": 0,
                                "cores": 0,
                                "memory": 0,
                                "persistent": 0,
                                "gpus": 0,
                            },
                            "external_user_groups": [],
                            "gpu_usage_supported": True,
                            "_links": {"self": {"href": "/api/v1/tenant/2"}},
                            "filesystem_mount_supported": True,
                            "tenant_enforcements": [],
                            "label": {
                                "name": "Demo Tenant",
                                "description": (
                                    "Demo Tenant for BlueData Clusters"
                                ),
                            },
                            "constraints_supported": True,
                            "tenant_storage_quota_supported": True,
                            "qos_multiplier": 1,
                        },
                    ]
                },
                "_links": {"self": {"href": "/api/v1/tenant"}},
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/tenant/1",
        response=MockResponse(
            json_data={
                "status": "ready",
                "features": {"ml_project": False, "kubernetes_access": False},
                "persistent_supported": True,
                "member_key_available": "all_admins",
                "quota": {},
                "cluster_isolation_supported": True,
                "inusequota": {
                    "disk": 0,
                    "cores": 0,
                    "memory": 0,
                    "persistent": 0,
                    "gpus": 0,
                },
                "external_user_groups": [],
                "gpu_usage_supported": True,
                "_links": {"self": {"href": "/api/v1/tenant/1"}},
                "filesystem_mount_supported": True,
                "tenant_enforcements": [],
                "label": {
                    "name": "Site Admin",
                    "description": ("Site Admin Tenant for BlueData clusters"),
                },
                "constraints_supported": False,
                "tenant_storage_quota_supported": False,
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/tenant/2",
        response=MockResponse(
            json_data={
                "status": "ready",
                "tenant_type": "docker",
                "features": {"ml_project": False, "kubernetes_access": False},
                "persistent_supported": True,
                "member_key_available": "all_admins",
                "quota": {},
                "cluster_isolation_supported": True,
                "inusequota": {
                    "disk": 0,
                    "cores": 0,
                    "memory": 0,
                    "persistent": 0,
                    "gpus": 0,
                },
                "external_user_groups": [],
                "gpu_usage_supported": True,
                "_links": {"self": {"href": "/api/v1/tenant/2"}},
                "filesystem_mount_supported": True,
                "tenant_enforcements": [],
                "label": {
                    "name": "Demo Tenant",
                    "description": "Demo Tenant for BlueData Clusters",
                },
                "constraints_supported": True,
                "tenant_storage_quota_supported": True,
                "qos_multiplier": 1,
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/tenant/100",
        response=MockResponse(
            json_data={},
            status_code=404,
            headers={},
            raise_for_status_flag=True,
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


from .base import BaseTestCase, MockResponse


def mockApiSetup():

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/user/",
        response=MockResponse(
            json_data={
                "_embedded": {
                    "users": [
                        {
                            "_links": {"self": {"href": "/api/v1/user/16"}},
                            "label": {"name": "csnow", "description": "chris"},
                            "is_group_added_user": False,
                            "is_external": False,
                            "is_service_account": False,
                            "default_tenant": "",
                            "is_siteadmin": False,
                        },
                        {
                            "_links": {"self": {"href": "/api/v1/user/5"}},
                            "label": {
                                "name": "admin",
                                "description": "BlueData Administrator",
                            },
                            "is_group_added_user": False,
                            "is_external": False,
                            "is_service_account": False,
                            "default_tenant": "/api/v1/tenant/1",
                            "is_siteadmin": True,
                        },
                    ]
                }
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/user/16/",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v1/user/16"}},
                "label": {"name": "csnow", "description": "chris"},
                "is_group_added_user": False,
                "is_external": False,
                "is_service_account": False,
                "default_tenant": "",
                "is_siteadmin": False,
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/user/123",
        response=MockResponse(
            json_data={
                "_links": {"self": {"href": "/api/v1/user/123"}},
                "purpose": "proxy",
            },
            status_code=200,
            headers={},
        ),
    )

    BaseTestCase.registerHttpGetHandler(
        url="https://127.0.0.1:8080/api/v1/user/999",
        response=MockResponse(
            text_data="Not found.",
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    BaseTestCase.registerHttpPostHandler(
        url="https://127.0.0.1:8080/api/v1/user",
        response=MockResponse(
            json_data={},
            status_code=200,
            headers={"location": ("/mock/api/user/1")},
        ),
    )

    BaseTestCase.registerHttpDeleteHandler(
        url="https://127.0.0.1:8080/api/v1/user/999",
        response=MockResponse(
            text_data="Not found.",
            json_data={},
            status_code=404,
            raise_for_status_flag=True,
            headers={},
        ),
    )

    BaseTestCase.registerHttpDeleteHandler(
        url="https://127.0.0.1:8080/api/v1/user/123",
        response=MockResponse(
            text_data="",
            json_data={},
            status_code=200,
            headers={},
        ),
    )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.


import json

import six
from mock import patch

from .base import BaseTestCase, MockResponse
from .lock_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestCLIList(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_yaml(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.lock.list()
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = """\
_embedded:
  external_locks: []
  internal_locks: []
_links:
  self:
    href: /api/v1/lock
locked: false"""

        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_json(self, mock_post, mock_get):

        self.maxDiff = None

        try:
            hpecp = self.cli.CLI()
            hpecp.lock.list(output="json")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = json.dumps(
            {
                "_links": {"self": {"href": "/api/v1/lock"}},
                "locked": False,
                "_embedded": {"internal_locks": [], "external_locks": []},
            }
        )
        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            expected_stderr = ""
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    def test_list_output_parameter_invalid(self, mock_post, mock_get):

        self.maxDiff = None

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.lock.list(output="garbage")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        self.assertEqual(stdout, expected_stdout)

        expected_stderr = "'output' parameter must be 'yaml' or 'json'"
        self.assertEqual(stderr, expected_stderr)


class TestCLIDelete(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    def test_delete(self, mock_post, mock_delete):

        try:
            hpecp = self.cli.CLI()
            hpecp.lock.delete(id="/api/v1/lock/1")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    def test_delete_all(self, mock_get, mock_post, mock_delete):

        try:
            hpecp = self.cli.CLI()
            hpecp.lock.delete_all()
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    def mocked_requests_get_locked(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/lock":
            return MockResponse(
                json_data={
                    "_links": {"self": {"href": "/api/v1/lock"}},
                    "locked": True,
                    "_embedded": {
                        "internal_locks": ["1"],
                        "external_locks": [],
                    },
                },
                status_code=200,
                headers=dict(),
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.get", side_effect=mocked_requests_get_locked)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.delete", side_effect=BaseTestCase.httpDeleteHandlers)
    def test_delete_all_timeout(self, mock_get, mock_post, mock_delete):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.lock.delete_all(timeout_secs=1)

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = "Could not delete locks."

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)


class TestCLICreate(BaseTestCase):
    def mocked_requests_get_locked(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/lock":
            return MockResponse(
                json_data={
                    "_links": {"self": {"href": "/api/v1/lock"}},
                    "locked": True,
                    "quiesced": True,
                    "_embedded": {
                        "internal_locks": ["1"],
                        "external_locks": [],
                    },
                },
                status_code=200,
                headers=dict(),
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("requests.get", side_effect=mocked_requests_get_locked)
    def test_create(self, mock_post, mock_get):

        try:
            hpecp = self.cli.CLI()
            hpecp.lock.create(reason="update")
        except Exception as e:
            # Unexpected Exception
            self.fail(e)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "/test_location/1"
        expected_stderr = ""

        self.assertEqual(
            stdout,
            expected_stdout,
            "Expected: '{}' Actual: '{}'".format(expected_stdout, stdout),
        )

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    def mocked_requests_post_with_exception(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={},
                status_code=200,
                headers={
                    "location": (
                        "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71"
                    )
                },
            )
        elif args[0] == "https://127.0.0.1:8080/api/v1/lock":
            return MockResponse(
                json_data={},
                status_code=400,
                raise_for_status_flag=True,
                headers={},
            )
        raise RuntimeError("Unhandle GET request: " + args[0])

    @patch("requests.post", side_effect=mocked_requests_post_with_exception)
    def test_create_with_exception(self, mock_post):

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.lock.create(reason="update")

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = ""
        expected_stderr = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        self.assertTrue(
            stderr.endswith(expected_stderr),
            "expected: `{}` actual: `{}`".format(expected_stderr, stderr),
        )



import tempfile

from mock import patch

from hpecp import ContainerPlatformClient
from hpecp.cli import base
from hpecp.exceptions import APIItemConflictException, APIItemNotFoundException
from hpecp.k8s_worker import K8sWorkerController, WorkerK8s, WorkerK8sStatus

from .base import BaseTestCase, get_client
from .k8s_worker_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestWorkers(BaseTestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_create_with_ssh_key_data(self, mock_get, mock_post):

        client = get_client()

        # Makes GET Request: https://127.0.0.1:8080/api/v2/worker/k8shost/
        worker_id = client.k8s_worker.create_with_ssh_key(
            ip="127.0.0.1", ssh_key_data="test ssh key"
        )

        self.assertEqual(worker_id, "/new/cluster/id")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_k8shosts(self, mock_get, mock_post):

        client = get_client()

        # Makes GET Request: https://127.0.0.1:8080/api/v2/worker/k8shost/
        workers = client.k8s_worker.list()

        # Test that json response is saved in each WorkerK8s object
        assert client.k8s_worker.list()[0].json is not None

        # Test WorkerK8sList subscriptable access and property setters
        assert workers[0].worker_id == 4
        assert workers[0].status == WorkerK8sStatus.unlicensed.name
        assert (
            workers[0].hostname == "ip-10-1-0-238.eu-west-2.compute.internal"
        )
        assert workers[0].ipaddr == "10.1.0.238"
        assert workers[0].href == "/api/v2/worker/k8shost/4"

        # Test WorkerK8sList iterators
        assert [worker.worker_id for worker in client.k8s_worker.list()] == [
            4,
            5,
        ]

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_k8shosts_with_setup_log(self, mock_get, mock_post):

        client = get_client()
        worker = client.k8s_worker.get(
            id="/api/v2/worker/k8shost/5", setup_log=True
        )

        self.assertEquals(
            worker.ipaddr,
            "10.1.0.186",
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_set_storage_invalid_worker_id(self, mock_get, mock_post):
        client = get_client()
        with self.assertRaisesRegexp(
            AssertionError,
            "'id' does not start with '/api/v2/worker/k8shost'",
        ):
            client.k8s_worker.set_storage(worker_id="garbage")

        with self.assertRaises(AssertionError):
            client.k8s_worker.set_storage(worker_id=123)

        with self.assertRaises(APIItemNotFoundException):
            client.k8s_worker.set_storage(worker_id="/api/v2/worker/k8shost/8")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_set_storage_no_disks(self, mock_get, mock_post):
        client = get_client()

        with self.assertRaises(AssertionError) as c:
            client.k8s_worker.set_storage(worker_id="/api/v2/worker/k8shost/5")
        self.assertEqual(
            str(c.exception),
            "'ephemeral_disks' must contain at least one disk",
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_set_storage_invalid_ephemeral_disks(self, mock_get, mock_post):
        client = get_client()

        with self.assertRaises(AssertionError) as c:
            client.k8s_worker.set_storage(
                worker_id="/api/v2/worker/k8shost/5",
                ephemeral_disks="garbage",
            )
        self.assertEqual(
            str(c.exception),
            "'ephemeral_disks' must be provided and and must be a list",
        )

        with self.assertRaises(AssertionError) as c:
            client.k8s_worker.set_storage(
                worker_id="/api/v2/worker/k8shost/5", ephemeral_disks=list()
            )
        self.assertEqual(
            str(c.exception),
            "'ephemeral_disks' must contain at least one disk",
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_set_storage_invalid_persistent_disks(self, mock_get, mock_post):
        client = get_client()
        _sample_ep_disks = ["/dev/nvme2n1", "/dev/nvme2n2"]

        with self.assertRaises(AssertionError) as c:
            client.k8s_worker.set_storage(
                worker_id="/api/v2/worker/k8shost/5",
                ephemeral_disks=_sample_ep_disks,
                persistent_disks="garbage",
            )
        self.assertEqual(str(c.exception), "'persistent_disks' must be a list")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_set_storage_only_ephemeral_disks(self, mock_get, mock_post):
        client = get_client()

        _sample_ep_disks = ["/dev/nvme2n1", "/dev/nvme2n2"]
        client.k8s_worker.set_storage(
            worker_id="/api/v2/worker/k8shost/5",
            ephemeral_disks=_sample_ep_disks,
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_cli(self, mock_get, mock_post):

        try:
            hpecp_cli = self.cli.CLI()
            hpecp_cli.k8sworker.set_storage(
                id="/api/v2/worker/k8shost/5",
                ephemeral_disks="abc,def",
                persistent_disks="ghi,jkl",
            )
        except Exception as e:
            self.fail("Unexpected exception: {}".format(e))


class TestCliCreate(BaseTestCase):
    def test_key_or_keycontent_provided(
        self,
    ):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8sworker.create_with_ssh_key(ip="127.0.0.1")

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = (
            "At least one of ssh_key or ssh_key_file must be provided"
        )

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )

    def test_key_and_keycontent_provided(
        self,
    ):

        hpecp_cli = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp_cli.k8sworker.create_with_ssh_key(
                ip="127.0.0.1", ssh_key="foobar", ssh_key_file="foobar"
            )

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = (
            "Either ssh_key or ssh_key_file must be provided, but not both."
        )

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("hpecp.k8s_worker")
    def test_with_only_ssh_key_content_provided(
        self, mock_post, mock_k8sworker
    ):

        """Test that the ssh key content provided by the 'ssh_key' parameter
        is passed to the library method 'create_with_ssh_key()'.
        """

        def mock_k8s_worker_get(*args, **kwargs):
            worker = WorkerK8s(
                {
                    "status": "bundle",
                    "approved_worker_pubkey": [],
                    "tags": [],
                    "hostname": "",
                    "ipaddr": "10.1.0.186",
                    "setup_log": (
                        "/var/log/bluedata/install/"
                        "k8shost_setup_10.1.0.186-"
                        "2020-4-26-18-49-10"
                    ),
                    "_links": {"self": {"href": "/api/v2/worker/k8shost/5"}},
                }
            )
            return worker

        def mock_create_with_ssh_key(*args, **kwargs):
            return "/api/v2/worker/k8shost/5"

        def mock_get_client():
            client = ContainerPlatformClient(
                username="",
                password="",
                api_host="",
                api_port=9090,
                use_ssl=True,
                verify_ssl=True,
                warn_ssl=True,
            )
            client.session_id = "ABC"
            client.k8s_worker.get = mock_k8s_worker_get
            client.k8s_worker.create_with_ssh_key = mock_create_with_ssh_key
            return client

        # support debugging if this test fails
        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp_cli = self.cli.CLI()

            # manually patch methods due to json serialization error
            # when using Mock or MagicMock
            base.get_client = mock_get_client

            hpecp_cli.k8sworker.create_with_ssh_key(
                ip="127.0.0.1",
                ssh_key="test_ssh_key",
            )

        stdout = self.out.getvalue().strip()

        self.assertEqual(stdout, "/api/v2/worker/k8shost/5")

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("hpecp.k8s_worker")
    def test_with_only_ssh_key_content_provided_raises_assertion_error(
        self, mock_post, mock_k8sworker
    ):

        with patch.object(
            K8sWorkerController,
            "create_with_ssh_key",
            side_effect=AssertionError("TEST_ASSERTION"),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp_cli = self.cli.CLI()
                hpecp_cli.k8sworker.create_with_ssh_key(
                    ip="127.0.0.1",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = "TEST_ASSERTION"

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("hpecp.k8s_worker")
    def test_with_only_ssh_key_content_provided_raises_conflict_exception(
        self, mock_post, mock_k8sworker
    ):

        with patch.object(
            K8sWorkerController,
            "create_with_ssh_key",
            side_effect=APIItemConflictException(
                message="APIItemConflictException.message",
                request_method="METHOD",
                request_url="URL",
            ),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp_cli = self.cli.CLI()
                hpecp_cli.k8sworker.create_with_ssh_key(
                    ip="127.0.0.1",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = "APIItemConflictException.message"

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("hpecp.k8s_worker")
    def test_with_only_ssh_key_content_provided_raises_general_exception(
        self, mock_post, mock_k8sworker
    ):

        with patch.object(
            K8sWorkerController,
            "create_with_ssh_key",
            side_effect=Exception("TEST_EXCEPTION"),
        ):
            with self.assertRaises(SystemExit) as cm:
                hpecp_cli = self.cli.CLI()
                hpecp_cli.k8sworker.create_with_ssh_key(
                    ip="127.0.0.1",
                    ssh_key="test_ssh_key",
                )

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_err = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(expected_err),
            "Expected: `{}`, Actual: `{}`".format(expected_err, stderr),
        )

    @patch(
        "requests.post", side_effect=BaseTestCase.httpPostHandlers
    )  # Login response
    def test_with_only_ssh_key_file_provided(self, mock_login_response):

        ssh_key_file = tempfile.NamedTemporaryFile(delete=True, mode="w")
        ssh_key_file.write("test_ssh_key_file_data")
        ssh_key_file.flush()

        def mock_k8s_worker_get(*args, **kwargs):
            worker = WorkerK8s(
                {
                    "status": "bundle",
                    "approved_worker_pubkey": [],
                    "tags": [],
                    "hostname": "",
                    "ipaddr": "10.1.0.186",
                    "setup_log": (
                        "/var/log/bluedata/install/"
                        "k8shost_setup_10.1.0.186-"
                        "2020-4-26-18-49-10"
                    ),
                    "_links": {"self": {"href": "/api/v2/worker/k8shost/5"}},
                }
            )
            return worker

        def mock_create_with_ssh_key(*args, **kwargs):
            return "/api/v2/worker/k8shost/5"

        def mock_get_client():
            client = ContainerPlatformClient(
                username="",
                password="",
                api_host="",
                api_port=9090,
                use_ssl=True,
                verify_ssl=True,
                warn_ssl=True,
            )
            client.session_id = "ABC"
            client.k8s_worker.get = mock_k8s_worker_get
            client.k8s_worker.create_with_ssh_key = mock_create_with_ssh_key
            return client

        # support debugging if this test fails
        with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
            hpecp_cli = self.cli.CLI()

            # manually patch methods due to json serialization error
            # when using Mock or MagicMock
            base.get_client = mock_get_client

            hpecp_cli.k8sworker.create_with_ssh_key(
                ip="127.0.0.1", ssh_key_file=ssh_key_file.name
            )

        stdout = self.out.getvalue().strip()

        self.assertEqual(stdout, "/api/v2/worker/k8shost/5")

        ssh_key_file.close()

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_ip_not_provided(self, mocked_requests_post):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8sworker.create_with_ssh_key(ssh_key="test data", ip=None)

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = "'ip' must be provided and must be a string"

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_ssh_key_not_a_string(self, mocked_requests_post):

        hpecp = self.cli.CLI()
        with self.assertRaises(SystemExit) as cm:
            hpecp.k8sworker.create_with_ssh_key(ssh_key=123, ip="127.0.0.1")

        self.assertEqual(cm.exception.code, 1)

        actual_err = self.err.getvalue().strip()
        expected_err = "'ssh_key_data' must be provided and must be a string"

        self.assertEqual(self.out.getvalue(), "", "stdout should be empty")

        self.assertTrue(
            actual_err.endswith(expected_err),
            "Actual stderr: `{}` Expected stderr: `{}`".format(
                actual_err, expected_err
            ),
        )


class TestCliStates(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_states(self, mock_post):

        self.maxDiff = None

        hpecp = self.cli.CLI()
        hpecp.k8sworker.statuses()

        stdout = self.out.getvalue().strip()

        expected_stdout = (
            "['bundle', 'installing', 'installed', 'ready', "
            "'unlicensed', 'configuring', 'configured', 'error', "
            "'sysinfo', 'unconfiguring', 'deleting', "
            "'storage_pending', 'storage_configuring', "
            "'storage_error']"
        )

        self.assertEqual(stdout, expected_stdout)


class TestCliSetStorage(BaseTestCase):
    @patch("hpecp.k8s_worker")
    def test_wwithout_ephemeral_storage_prints_err(self, mock_k8sworker):

        with patch.object(
            K8sWorkerController,
            "set_storage",
            return_value=None,
        ):

            with self.assertRaises(SystemExit) as cm:
                try:
                    hpecp_cli = self.cli.CLI()
                    hpecp_cli.k8sworker.set_storage(
                        id="12345",
                        ephemeral_disks=None,
                        persistent_disks=None,
                    )
                except Exception as e:
                    self.fail("Unexpected exception: {}".format(e))

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        exptected_stderr = "'ephemeral_disks' must be provided"

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(exptected_stderr),
            "Expected: `{}` Actual: `{}`".format(exptected_stderr, stderr),
        )

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    @patch("hpecp.k8s_worker")
    def test_with_exception(self, mock_post, mock_k8sworker):

        with patch.object(
            K8sWorkerController,
            "set_storage",
            side_effect=Exception("TEST_EXCEPTION"),
        ):

            with self.assertRaises(SystemExit) as cm:
                try:
                    hpecp_cli = self.cli.CLI()
                    hpecp_cli.k8sworker.set_storage(
                        id="12345",
                        ephemeral_disks="abc,def",
                        persistent_disks="ghi,jkl",
                    )
                except Exception as e:
                    self.fail("Unexpected exception: {}".format(e))

        self.assertEqual(cm.exception.code, 1)

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        exptected_stderr = (
            "Unknown error. To debug run with env var LOG_LEVEL=DEBUG"
        )

        self.assertEqual(stdout, "")
        self.assertTrue(
            stderr.endswith(exptected_stderr),
            "Expected: `{}` Actual: `{}`".format(exptected_stderr, stderr),
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from mock import patch

from hpecp.exceptions import APIItemNotFoundException

from .base import BaseTestCase, get_client
from .tenant_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


class TestTentants(BaseTestCase):
    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_tenant_list(self, mock_get, mock_post):

        client = get_client()

        # Makes GET Request: https://127.0.0.1:8080/api/v1/tenant
        tenants = client.tenant.list()

        # Test that json response is saved in each Tenant object
        self.assertIsNotNone(client.tenant.list()[0].json)

        # Test TenantList subscriptable access and Tenant property setters
        self.assertEqual(tenants[0].id, "/api/v1/tenant/1")
        self.assertEqual(tenants[0].status, "ready")
        self.assertEqual(tenants[0].name, "Site Admin")
        self.assertEqual(tenants[0].tenant_type, "docker")
        self.assertEqual(
            tenants[0].description, "Site Admin Tenant for BlueData clusters"
        )

        # Test TenantList iterators
        self.assertEqual(
            [tenant.id for tenant in client.tenant.list()],
            ["/api/v1/tenant/1", "/api/v1/tenant/2"],
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_tenant_id_format(self, mock_get, mock_post):
        client = get_client()

        with self.assertRaisesRegexp(
            AssertionError,
            ("'id' does not start with '/api/v1/tenant/'"),
        ):
            client.tenant.get("garbage")

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get_tenant(self, mock_get, mock_post):
        tenant = get_client().tenant.get("/api/v1/tenant/1")
        self.assertEqual(tenant.id, "/api/v1/tenant/1")

        tenant = get_client().tenant.get("/api/v1/tenant/2")
        self.assertEqual(tenant.id, "/api/v1/tenant/2")

        with self.assertRaises(APIItemNotFoundException):
            get_client().tenant.get("/api/v1/tenant/100")



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import json
import os
import sys
import tempfile
from textwrap import dedent
from unittest import TestCase

import six
from mock import mock, mock_open, patch

from hpecp.cli import base
from hpecp.gateway import Gateway

from .base import BaseTestCase, MockResponse
from .cli_mock_api_responses import mockApiSetup

# setup the mock data
mockApiSetup()


if six.PY2:
    # from io import BytesIO as StringIO  # noqa: F811
    from test.test_support import EnvironmentVarGuard
else:
    # from io import StringIO
    from test.support import EnvironmentVarGuard

try:
    from imp import reload
except Exception:
    from importlib import reload


class TestCLI(BaseTestCase):
    def test_config_file_missing(self):
        def get_config_file():
            return "this_file_should_not_exist"

        with self.assertRaises(SystemExit) as cm:
            base.get_config_file = get_config_file
            base.get_client()

        self.assertEqual(cm.exception.code, 1)

        self.assertEqual(self.out.getvalue(), "")

        self.assertEqual(
            self.err.getvalue(),
            "Could not find configuration file 'this_file_should_not_exist'\n",
        )

    def test_configure_cli_writes_hpecp_conf(self):

        if six.PY2:
            builtins_name = "__builtin__.open"
        else:
            builtins_name = "builtins.open"

        with patch(builtins_name, mock_open()) as m:

            # mock the input capture to simulate user input
            # TODO: we want to send different data for each parameter
            six.moves.input = lambda *args: ("1234")

            hpecp = self.cli.CLI()
            hpecp.configure_cli()

        handle = m()
        handle.write.assert_has_calls(
            [
                mock.call("[default]\n"),
                mock.call("api_host = 1234\n"),
                mock.call("api_port = 1234\n"),
                mock.call("use_ssl = 1234\n"),
                mock.call("verify_ssl = 1234\n"),
                mock.call("warn_ssl = 1234\n"),
                mock.call("username = 1234\n"),
                mock.call("password = 1234\n"),
                mock.call("\n"),
            ]
        )

    def test_configure_cli_reads_hpecp_conf(self):

        mock_data = dedent(
            """                [default]
                api_host = mock_host
                api_port = 9999
                use_ssl = True
                verify_ssl = False
                warn_ssl = True
                username = admin
                password = admin123"""
        ).encode("utf8")

        if six.PY2:
            builtins_name = "__builtin__.open"
        else:
            builtins_name = "builtins.open"

        with patch(builtins_name, mock_open(read_data=mock_data)):
            with patch("os.path.exists") as os_path_exists:

                # instruct the CLI that the mock file is actually
                # ~/.hpecp.conf
                os_path_exists.return_value = True

                # mock the input capture to simulate user input
                # TODO: we want to send different data for each configuration
                # parameter
                six.moves.input = lambda *args: ("1234")

                hpecp = self.cli.CLI()
                hpecp.configure_cli()

                self.assertIn(
                    "Controller API Host [mock_host]:", self.out.getvalue()
                )
                self.assertIn(
                    "Controller API Port [9999]:", self.out.getvalue()
                )
                # TODO check all values

    def test_autocomplete_bash(self):

        if six.PY3:
            try:
                hpecp = self.cli.CLI()
                hpecp.autocomplete.bash()
            except Exception as e:
                # Unexpected Exception
                self.fail(e)


class TestCLIConfig(TestCase):
    def setUp(self):
        sys.path.insert(0, os.path.abspath("../../"))

    def test_configure_cli_reads_hpecp_conf_user_provided_profile(self):

        # FIXME: why is this broken on 2.7?
        if six.PY2:
            return

        mock_data = dedent(
            """                [default]
                api_host = mock_host
                api_port = 9999
                use_ssl = True
                verify_ssl = False
                warn_ssl = True
                username = admin
                password = admin123

                [tenant1]
                api_host = tenant_mock_host
                username = tenant-admin
                password = tenant-password
                tenant = /api/v1/tenant/2"""
        ).encode("utf8")

        if six.PY2:
            builtins_name = "__builtin__.open"
        else:
            builtins_name = "builtins.open"

        with patch.dict("os.environ", {"PROFILE": "tenant1"}):
            with patch(builtins_name, mock_open(read_data=mock_data)):
                with patch("os.path.exists") as os_path_exists:

                    self.assertEqual(base.get_profile(), "tenant1")

                    # instruct the CLI that the mock file is actually
                    # ~/.hpecp.conf
                    os_path_exists.return_value = True

                    hpecp_client = base.get_client(start_session=False)

                    # this should be from the [default] section
                    self.assertEqual(hpecp_client.api_port, 9999)

                    # this should be from the [tenant1] section
                    self.assertEqual(hpecp_client.api_host, "tenant_mock_host")
                    self.assertEqual(hpecp_client.username, "tenant-admin")
                    self.assertEqual(
                        hpecp_client.tenant_config, "/api/v1/tenant/2"
                    )


class TestBaseProxy(BaseTestCase):
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_list_with_invalid_column(self, mock_post):

        with self.assertRaises(SystemExit) as cm:
            with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
                hpecp_cli = self.cli.CLI()

                # we could have used any of the proxies implementing
                # BaseProxy - here we arbitrarily chosen GatewayProxy
                hpecp_cli.gateway.list(columns=["not_a_column"])

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(error, "Unknown column 'not_a_column'.")

        self.assertEqual(cm.exception.code, 1)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_list_with_invalid_columns_list(self, mock_post):

        with self.assertRaises(SystemExit) as cm:
            with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
                hpecp_cli = self.cli.CLI()

                # we could have used any of the proxies implementing
                # BaseProxy - here we arbitrarily chosen GatewayProxy
                hpecp_cli.gateway.list(columns=123)

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(error, "'columns' parameter must be a list.")

        self.assertEqual(cm.exception.code, 1)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_list_with_invalid_output_param(self, mock_post):

        with self.assertRaises(SystemExit) as cm:
            with patch.dict("os.environ", {"LOG_LEVEL": "DEBUG"}):
                hpecp_cli = self.cli.CLI()

                # we could have used any of the proxies implementing
                # BaseProxy - here we arbitrarily chosen GatewayProxy
                try:
                    hpecp_cli.gateway.list(output="invalid_output_value")
                except Exception as e:
                    self.fail(e)

        output = self.out.getvalue().strip()
        self.assertEqual(output, "")

        error = self.err.getvalue().strip()
        self.assertEqual(
            error,
            "When providing the --columns param, the --output param must be 'table' or 'text'",
        )

        self.assertEqual(cm.exception.code, 1)


class TestCLIUsingCfgFileEnvVar(TestCase):
    def test_hpe_config_file_var(self):

        from hpecp.cli.base import get_config_file

        dummy_filepath = "/not/a/real/dir/not_a_real_file"

        env = EnvironmentVarGuard()
        env.set("HPECP_CONFIG_FILE", dummy_filepath)

        with env:
            sys.path.insert(0, os.path.abspath("../../"))
            from bin import cli

            # reload cli module with mock env
            reload(cli)

            self.assertEqual(dummy_filepath, get_config_file())


class TestCLIHttpClient(BaseTestCase):
    def mocked_requests_failed_login(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/api/v1/login":
            return MockResponse(
                json_data={"foo": "bar"},
                text_data='{"foo":"bar"}',
                raise_connection_error=True,
                status_code=500,
                headers=dict(),
            )
        raise RuntimeError("Unhandle POST request: " + args[0])

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=mocked_requests_failed_login)
    def test_get_failed_login(self, mock_get, mock_post):

        # TODO move this to TestCLI class

        with self.assertRaises(SystemExit) as cm:
            hpecp = self.cli.CLI()
            hpecp.httpclient.get("/")  # our mock raises an exception on login

        self.assertEqual(cm.exception.code, 1)

        self.assertEqual(self.out.getvalue().strip(), "")

        actual_err = self.err.getvalue().strip()
        expected_err = "Could not connect to the controller."

        # coverage seems to populate standard error (issues 93)
        self.assertTrue(
            self.err.getvalue().strip().endswith(expected_err),
            "Expected: `{}` Actual: `{}`".format(expected_err, actual_err),
        )

    @patch("requests.get", side_effect=BaseTestCase.httpGetHandlers)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_get(self, mock_get, mock_post):

        hpecp = self.cli.CLI()
        hpecp.httpclient.get(url="/some/url")

        self.assertEqual(self.out.getvalue(), '{"foo": "bar"}\n')

    def mocked_requests_delete(*args, **kwargs):
        if args[0] == "https://127.0.0.1:8080/some/url":
            return MockResponse(
                json_data={"foo": "bar"},
                text_data='{"foo":"bar"}',
                status_code=200,
                headers=dict(),
            )
        raise RuntimeError("Unhandle DELETE request: " + args[0])

    @patch("requests.delete", side_effect=mocked_requests_delete)
    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_delete(self, mock_delete, mock_post):

        hpecp = self.cli.CLI()
        hpecp.httpclient.delete(
            url="/some/url",
        )

        self.assertEqual(self.out.getvalue(), "")

    def test_post(self):

        with patch("requests.post") as mock_requests:
            mock_requests.side_effect = BaseTestCase.httpPostHandlers

            with tempfile.NamedTemporaryFile() as json_file:
                json_file.write(json.dumps({"abc": "def"}).encode("utf-8"))
                json_file.flush()

                hpecp = self.cli.CLI()
                hpecp.httpclient.post(
                    url="/some/url", json_file=json_file.name
                )

            mock_requests.assert_called_with(
                "https://127.0.0.1:8080/some/url",
                data='{"abc": "def"}',
                headers={
                    "content-type": "application/json",
                    "X-BDS-SESSION": "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71",
                    "accept": "application/json",
                    "cache-control": "no-cache",
                },
                verify=False,
            )

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "{'mock_data': True}"
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)

    @patch("requests.post", side_effect=BaseTestCase.httpPostHandlers)
    def test_put(self, mock_post):

        with patch("requests.put") as mock_requests:
            mock_requests.side_effect = BaseTestCase.httpPutHandlers

            with tempfile.NamedTemporaryFile() as json_file:
                json_file.write(json.dumps({"abc": "def"}).encode("utf-8"))
                json_file.flush()

                hpecp = self.cli.CLI()
                hpecp.httpclient.put(url="/some/url", json_file=json_file.name)

            mock_requests.assert_called_with(
                "https://127.0.0.1:8080/some/url",
                data='{"abc": "def"}',
                headers={
                    "content-type": "application/json",
                    "X-BDS-SESSION": "/api/v1/session/df1bfacb-xxxx-xxxx-xxxx-c8f57d8f3c71",
                    "accept": "application/json",
                    "cache-control": "no-cache",
                },
                verify=False,
            )

        stdout = self.out.getvalue().strip()
        stderr = self.err.getvalue().strip()

        expected_stdout = "{'mock_data': True}"
        expected_stderr = ""

        self.assertEqual(stdout, expected_stdout)

        # coverage seems to populate standard error on PY3 (issues 93)
        if six.PY2:
            self.assertEqual(stderr, expected_stderr)


class TestBash(BaseTestCase):
    def test_get_metadata(self):

        self.maxDiff = None

        hpecp_cli = self.cli.CLI()
        (modules, columns) = hpecp_cli.autocomplete._get_metadata()

        self.assertEquals(columns["gateway"], Gateway.all_fields)

        self.assertEquals(
            list(modules["gateway"].keys()),
            [
                "wait_for_state",
                "states",
                "list",
                "get",
                "delete",
                "create_with_ssh_key",
            ],
            "Keys should be in reverse alphabetical order for autocompletion",
        )

        # test the introspection of CLI gateway.list() parameter names
        self.assertEquals(
            modules["gateway"]["list"],
            ["--output", "--columns", "--query"],
        )
